// SVN.cpp : Implementation of SVN

#include "stdafx.h"
#include "SVN.h"


struct log_msg_baton
{
  const char *message;
  const char *base_dir;
};

// SVN


STDMETHODIMP SVN::GetFileStatus(/* [in] */ VARIANT_BOOL update, /* [in] */ BSTR path, /* [out] */ IDispatch** StatusObj)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	const svn_item_t*		item;
	svn_revnum_t			youngest;

	CString p = CString(path);
	preparePath(p);

	youngest = SVN_INVALID_REVNUM;

	Err = svn_client_status (&statushash,
							&youngest,
							(LPCTSTR)p,
							auth_obj,
							0,
							1,
							update,
							1,
							pool);

	// Error present if function is not under version control
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	// Convert the unordered hash to an ordered, sorted array
	statusarray = apr_hash_sorted_keys (statushash,
										svn_sort_compare_items_as_paths,
										pool);

	//only the first entry is needed (no recurse)
	item = &APR_ARRAY_IDX (statusarray, 0, const svn_item_t);
	status = (svn_wc_status_t *) item->value;


	CComObject<SVNStatus>* pStatus = NULL;
	HRESULT hr = CComObject<SVNStatus>::CreateInstance(&pStatus);
	if (FAILED(hr))
	{
		//svn_pool_destroy (localpool);
		return hr;
	}

	pStatus->text_status = static_cast<svn_statuses>(status->text_status);
	pStatus->prop_status = static_cast<svn_statuses>(status->prop_status);
	pStatus->locked = status->locked;
	pStatus->repo_prop_status = static_cast<svn_statuses>(status->repos_prop_status);
	pStatus->repo_text_status = static_cast<svn_statuses>(status->repos_text_status);

	pStatus->name = CString(status->entry->name);
	pStatus->revision = status->entry->revision;
	pStatus->url = CString(status->entry->url);
	pStatus->repository = CString(status->entry->repos);
	
	/* State information */
	pStatus->schedule = static_cast<svn_schedule>(status->entry->schedule); //(svn_schedule)status->entry->schedule;
	pStatus->copied = status->entry->copied;
	pStatus->deleted = status->entry->deleted;

	pStatus->copyfrom_rev = status->entry->copyfrom_rev;
	pStatus->copyfrom_url = CString(status->entry->copyfrom_url);

	pStatus->conflict_old = CString(status->entry->conflict_old);
	pStatus->conflict_new = CString(status->entry->conflict_new);
	pStatus->conflict_wrk = CString(status->entry->conflict_wrk);
	//const char *prejfile;          /* property reject file */

	/* Timestamps (0 means no information available) */
	if (status->entry->text_time == 0)
		pStatus->text_time = COleDateTime(2000,1,1,0,0,0);
	else
		pStatus->text_time = COleDateTime((__time64_t)status->entry->text_time/1000000L);
	if (status->entry->prop_time == 0)
		pStatus->prop_time = COleDateTime(2000,1,1,0,0,0);
	else
		pStatus->prop_time = COleDateTime((__time64_t)status->entry->prop_time/1000000L);
	/* Checksum.  Optional; can be NULL for backwards compatibility. */
	//const char *checksum;          /* base64-encoded checksum for the
									//untranslated text base file. */

	/* "Entry props" */
	pStatus->cmt_rev = status->entry->cmt_rev;
	if (status->entry->cmt_date == 0)
		pStatus->cmt_date = COleDateTime(2000,1,1,0,0,0);
	else
		pStatus->cmt_date = COleDateTime((__time64_t)status->entry->cmt_date/1000000L);
	pStatus->cmt_author = CString(status->entry->cmt_author);
	
	

	//get the IDispatch interface to return to the caller
	CComQIPtr<IDispatch, &IID_IDispatch> pIDispatch(pStatus);
	pStatus->AddRef();
	if (!pIDispatch)
	{
		//svn_pool_destroy (localpool);
		return E_FAIL;
	}	

	*StatusObj = pStatus;

	//svn_pool_destroy (localpool);
	
	return S_OK;

}


STDMETHODIMP SVN::GetProperties(/* [in] */ BSTR path, /* [out] */ IDispatch** PropertiesObj)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CComObject<SVNProperties>* pProp = NULL;
	HRESULT hr = CComObject<SVNProperties>::CreateInstance(&pProp);
	if (FAILED(hr))
	{
		//svn_pool_destroy (localpool);
		return hr;
	}
	//get the IDispatch interface to return to the caller
	CComQIPtr<IDispatch, &IID_IDispatch> pIDispatch(pProp);
	pProp->AddRef();
	if (!pIDispatch)
	{
		//svn_pool_destroy (localpool);
		return E_FAIL;
	}
	hr = pProp->Create(path);

	if (FAILED(hr))
	{
		//svn_pool_destroy (localpool);
		pProp->Release();
		return hr;
	}

	*PropertiesObj = pProp;

	//svn_pool_destroy (localpool);
	
	return S_OK;
}

STDMETHODIMP SVN::Checkout(/* [in] */ BSTR moduleName, /* [in] */  BSTR destPath, /* [in] */ LONG revision, /* [in] */ VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString modName = CString(moduleName);
	preparePath(modName);
	CString path = CString(destPath);
	preparePath(path);


	Err = svn_client_checkout (notify,
								(void *)this,
								auth_obj,
								(LPCTSTR)modName,
								(LPCTSTR)path,
								getRevision (revision),
								recurse,
								NULL,
								pool);

	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);

	return S_OK;
}

STDMETHODIMP SVN::Remove(BSTR path, VARIANT_BOOL force)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);

	svn_client_commit_info_t *commit_info = NULL;

	Err = svn_client_delete (&commit_info, (LPCTSTR)urlPath, NULL, force,
							auth_obj,
							&svn_cl__get_log_message,		//dummy. we don't use that callback
							logMessage (NULL),
							notify, 
							(void *)this, 
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Revert(BSTR path, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);
	Err = svn_client_revert ((LPCTSTR)urlPath, recurse, notify, 
							(void *)this, pool);

	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}


STDMETHODIMP SVN::Add(BSTR path, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);
	Err = svn_client_add ((LPCTSTR)urlPath, recurse, notify, 
						(void *)this, pool);

	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Update(BSTR path, LONG revision, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);
	Err = svn_client_update (auth_obj,
							(LPCTSTR)urlPath,
							NULL,
							getRevision (revision),
							recurse,
							notify,
							(void *)this,
							pool);

	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}


STDMETHODIMP SVN::Commit(BSTR path, BSTR message, VARIANT_BOOL recurse, LONG* revision)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);
	svn_client_commit_info_t *commit_info = NULL;
	svn_revnum_t revnum = SVN_INVALID_REVNUM;

	Err = svn_client_commit (&commit_info, notify, 
							(void *)this, 
							auth_obj, target ((LPCTSTR)urlPath), 
							&svn_cl__get_log_message, 
							logMessage ((LPCTSTR)CString(message)), NULL, revnum, 
							!recurse,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	if(commit_info && SVN_IS_VALID_REVNUM (commit_info->revision))
		*revision = commit_info->revision;
	else
		*revision = -1;

	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Copy(BSTR srcPath, BSTR destPath, LONG revision)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString src = CString(srcPath);
	preparePath(src);
	CString dest = CString(destPath);
	preparePath(dest);

	svn_client_commit_info_t *commit_info = NULL;

	Err = svn_client_copy (&commit_info,
							(LPCTSTR)src,
							getRevision (revision),
							(LPCTSTR)dest,
							NULL,
							auth_obj,
							&svn_cl__get_log_message,
							logMessage (NULL),
							notify, 
							(void *)this,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Move(BSTR srcPath, BSTR destPath, VARIANT_BOOL force)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString src = CString(srcPath);
	preparePath(src);
	CString dest = CString(destPath);
	preparePath(dest);

	svn_client_commit_info_t *commit_info = NULL;

	Err = svn_client_move (&commit_info,
							(LPCTSTR)src,
							getRevision (-1),
							(LPCTSTR)dest,
							force,
							auth_obj,
							&svn_cl__get_log_message,
							logMessage (NULL),
							notify,
							(void *)this,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::MakeDir(BSTR path, BSTR message)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);

	svn_client_commit_info_t *commit_info = NULL;

	Err = svn_client_mkdir (&commit_info,
							(LPCTSTR)urlPath,
							auth_obj,
							&svn_cl__get_log_message,
							logMessage ((LPCTSTR)CString(message)),
							notify,
							(void *)this,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::CleanUp(BSTR path)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);
	Err = svn_client_cleanup ((LPCTSTR)urlPath, pool);

	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Resolve(BSTR path, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString urlPath = CString(path);
	preparePath(urlPath);
	Err = svn_client_resolve ((LPCTSTR)urlPath,
							notify,
							(void *)this,
							recurse,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Export(BSTR srcPath, BSTR destPath, LONG revision)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString src = CString(srcPath);
	preparePath(src);
	CString dest = CString(destPath);
	preparePath(dest);

	Err = svn_client_export ((LPCTSTR)src,
							(LPCTSTR)dest,
							getRevision (revision),
							auth_obj,
							notify,
							(void *)this,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Switch(BSTR path, BSTR url, LONG revision, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString src = CString(path);
	preparePath(src);
	CString dest = CString(url);
	preparePath(dest);

	Err = svn_client_switch (auth_obj,
							(LPCTSTR)src,
							(LPCTSTR)dest,
							getRevision (revision),
							recurse,
							notify,
							(void *)this,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Import(BSTR path, BSTR url, BSTR newEntry, BSTR message, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString src = CString(path);
	preparePath(src);
	CString dest = CString(url);
	preparePath(dest);

	svn_client_commit_info_t *commit_info = NULL;

	Err = svn_client_import (&commit_info,
							notify,
							(void *)this,
							auth_obj,   
							(LPCTSTR)src,
							(LPCTSTR)dest,
							(LPCTSTR)CString(newEntry),
							&svn_cl__get_log_message,
							logMessage ((LPCTSTR)CString(message)),
							NULL,
							SVN_INVALID_REVNUM,
							!recurse,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::Merge(BSTR path1, LONG revision1, BSTR path2, LONG revision2, BSTR localPath, VARIANT_BOOL force, VARIANT_BOOL recurse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	CString p1 = CString(path1);
	preparePath(p1);
	CString p2 = CString(path2);
	preparePath(p2);
	CString local = CString(localPath);
	preparePath(local);

	Err = svn_client_merge (notify,
							(void *)this,
							auth_obj,
							(LPCTSTR)p1,
							getRevision (revision1),
							(LPCTSTR)p2,
							getRevision (revision2),
							(LPCTSTR)local,
							recurse,
							force,
							pool);
	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}

STDMETHODIMP SVN::ReceiveLog(BSTR path, LONG revisionStart, LONG revisionEnd, VARIANT_BOOL reverse)
{
	//apr_pool_t *			localpool;
	//localpool = svn_pool_create (NULL);

	svn_opt_revision_t revEnd;
	memset (&revEnd, 0, sizeof (revEnd));
	revEnd.kind = svn_opt_revision_number;
	revEnd.value.number = revisionEnd;

	Err = svn_client_log (auth_obj, target ((LPCTSTR)CString(path)), 
						getRevision (revisionStart), 
						&revEnd, 
						reverse,		// not reverse by default
						true,			// strict by default (not showing cp info)
						logReceiver,	// log_message_receiver
						(void *)this, pool);

	if(Err != NULL)
	{
		//svn_pool_destroy (localpool);
		return Error(Err->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}
	//svn_pool_destroy (localpool);
	return S_OK;
}







svn_error_t* SVN::logReceiver(void* baton, 
								apr_hash_t* ch_paths, 
								svn_revnum_t rev, 
								const char* author, 
								const char* date, 
								const char* msg, 
								apr_pool_t* pool)
{
	svn_error_t * error = NULL;

	SVN * svn = (SVN *)baton;
	svn->Log(rev, CString(author).AllocSysString(), CString(date).AllocSysString(), CString(msg).AllocSysString());

	return error;
}

void SVN::notify( void *baton,
					const char *path,
					svn_wc_notify_action_t action,
					svn_node_kind_t kind,
					const char *mime_type,
					svn_wc_notify_state_t content_state,
					svn_wc_notify_state_t prop_state,
					svn_revnum_t revision)
{
	SVN * svn = (SVN *)baton;
	svn->Notify(CString(path).AllocSysString(), (notify_action)action, (node_kind)kind,
			CString(mime_type).AllocSysString(), (notify_state)content_state, (notify_state)prop_state, revision);
}

svn_error_t* SVN::prompt(char **info, const char *prompt, svn_boolean_t hide, void *baton, apr_pool_t *pool)
{
	SVN * svn = (SVN *)baton;
	
	BSTR infostring = ::SysAllocStringLen(NULL, 1024);
	svn->Prompt(/* [out] */ &infostring, /* [in] */ CString(prompt).AllocSysString(),/* [in] */ hide);
	CString temp = CString(infostring);
	::SysFreeString(infostring);
	SVN_ERR (svn_utf_cstring_to_utf8 ((const char **)info, (LPCTSTR)temp, NULL, pool));
	//*info = apr_pstrdup(pool,(LPCTSTR)temp);
	return SVN_NO_ERROR;
}

void * SVN::logMessage (const char * message, char * baseDirectory)
{
	log_msg_baton* baton = (log_msg_baton *) apr_palloc (pool, sizeof (*baton));

	baton->message = message;
	baton->base_dir = baseDirectory ? baseDirectory : ".";

	return baton;
}


svn_opt_revision_t*	SVN::getRevision (long revNumber)
{
	if(revNumber == -1)
		rev.kind = svn_opt_revision_unspecified;
	else
		rev.kind = svn_opt_revision_number;

	rev.value.number = revNumber;

	return &rev;
}

void	SVN::preparePath(CString &path)
{
	path.Trim();
	path.TrimRight("/\\");			//remove trailing slashes
	path.Replace('\\','/');
}

svn_error_t* svn_cl__get_log_message (const char **log_msg,
						apr_array_header_t * commit_items,
						void *baton, apr_pool_t * pool)
{
	log_msg_baton *lmb = (log_msg_baton *) baton;

	if (lmb->message)
	{
		*log_msg = apr_pstrdup (pool, lmb->message);
	}

	return SVN_NO_ERROR;
}

apr_array_header_t * SVN::target (const char * path)
{
	CString p = CString(path);
	apr_array_header_t *targets = apr_array_make (pool,
												DEFAULT_ARRAY_SIZE,
												sizeof (const char *));

	int curPos= 0;

	CString resToken= p.Tokenize(";",curPos);
	while (resToken != "")
	{
		const char * target = apr_pstrdup (pool, (LPCTSTR)resToken);
		(*((const char **) apr_array_push (targets))) = target;
		resToken= p.Tokenize(";",curPos);
	};
	return targets;
}



