// SVNProperties.cpp : Implementation of SVNProperties

#include "stdafx.h"
#include "SVNProperties.h"


// SVNProperties

svn_error_t*	SVNProperties::Refresh()
{
	svn_error_t*	error = NULL;

	error = svn_client_proplist (&props,
								(LPCTSTR)filePath, 
								false,	//recurse
								pool);
	if(error != NULL)
		return error;

	propCount = 0;

	for (int j = 0; j < props->nelts; j++)
	{
		svn_client_proplist_item_t *item = ((svn_client_proplist_item_t **)props->elts)[j];

		const char *node_name_native;
		error = svn_utf_cstring_from_utf8_stringbuf (&node_name_native,
												item->node_name,
												pool);

		if (error != NULL)
			return error;

		apr_hash_index_t *hi;

		for (hi = apr_hash_first (pool, item->prop_hash); hi; hi = apr_hash_next (hi))
		{
			propCount++;
		} 
	}
	return NULL;
}

STDMETHODIMP SVNProperties::Create(BSTR path)
{
	apr_pool_t *			pool;
	pool = svn_pool_create (NULL);

	filePath = CString(path);
	svn_error_t*	error = NULL;
	char*			truepath;

	propCount = 0;

	filePath.Trim();
	filePath.TrimRight("/\\");			//remove trailing slashes

	//change path format from '\' to '/'
	apr_filepath_merge (&truepath, "", (LPCTSTR)filePath,
                                        APR_FILEPATH_TRUENAME, pool);


	filePath = CString(truepath);

	//get the property list
	error = Refresh();
	if (error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	svn_pool_destroy (pool);
	return S_OK;
}

STDMETHODIMP SVNProperties::get_Count(LONG* pVal)
{
	if (!pVal)
		return E_POINTER;
	*pVal = propCount;
	return S_OK;
}

STDMETHODIMP SVNProperties::Item(LONG index, IDispatch** prop)
{
	apr_pool_t *			pool;
	pool = svn_pool_create (NULL);

	const void *key;
	const char *key_native;
	void *val;
	const svn_string_t *propval;
	const char *node_name_native;
	const char *pname_utf8;
	svn_error_t*	error = NULL;

	if (props == NULL)
	{
		return Error("not initialized!",0,NULL,this->GetObjectCLSID(),E_FAIL);
		svn_pool_destroy (pool);
	}
	if (index >= propCount)
	{
		svn_pool_destroy (pool);
		return Error("index is out of range",0,NULL,this->GetObjectCLSID(),E_INVALIDARG);
	}
	CComObject<SVNProperty>* pProp = NULL;
	HRESULT hr = CComObject<SVNProperty>::CreateInstance(&pProp);
	if (FAILED(hr))
	{
		svn_pool_destroy (pool);
		return hr;
	}

	long ind = 0;

	for (int j = 0; j < props->nelts; j++)
	{
		svn_client_proplist_item_t *item = ((svn_client_proplist_item_t **)props->elts)[j];

		error = svn_utf_cstring_from_utf8_stringbuf (&node_name_native,
													item->node_name,
													pool);
		if (error != NULL)
		{
			svn_pool_destroy (pool);
			return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
		}

		apr_hash_index_t *hi;

		for (hi = apr_hash_first (pool, item->prop_hash); hi; hi = apr_hash_next (hi))
		{
			if(ind++ != index)
				continue;

			apr_hash_this (hi, &key, NULL, &val);
			error = svn_utf_cstring_from_utf8 (&key_native, (char *)key, pool);
			if (error != NULL)
			{
				svn_pool_destroy (pool);
				return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
			}

			propval = (svn_string_t *)val;

			//If this is a special Subversion property, it is stored as
			//UTF8, so convert to the native format.
			svn_utf_cstring_to_utf8 (&pname_utf8, key_native, NULL, pool);
			if (svn_prop_is_svn_prop (pname_utf8))
			{
				error = svn_utf_string_from_utf8 (&propval, propval, pool);
				if (error != NULL)
				{
					svn_pool_destroy (pool);
					return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
				}
			}
		} 
	}

	pProp->name = CString(key_native);
	pProp->value = CString(propval->data);

	//get the IDispatch interface to return to the caller
	CComQIPtr<IDispatch, &IID_IDispatch> pIDispatch(pProp);
	pProp->AddRef();
	if (!pIDispatch)
	{
		svn_pool_destroy (pool);
		return E_FAIL;
	}

	*prop = pProp;

	svn_pool_destroy (pool);
	return S_OK;
}

STDMETHODIMP SVNProperties::Add(ISVNProperty* prop, VARIANT_BOOL recurse)
{
	apr_pool_t *			pool;
	pool = svn_pool_create (NULL);

	const char*		pname_utf8;
	svn_error_t*	error = NULL;
	svn_string_t*	pval;

	BSTR			pname;
	BSTR			pv;
	prop->get_Name(&pname);
	prop->get_Value(&pv);
	CString propname = CString(pname);
	CString propvalue = CString(pv);
	::SysFreeString(pname);
	::SysFreeString(pv);

	pval = svn_string_create ((LPCTSTR)propvalue, pool);

	error = svn_utf_cstring_to_utf8 (&pname_utf8, (LPCTSTR)propname, NULL, pool);
	if(error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	error = svn_client_propset (pname_utf8, pval, (LPCTSTR)filePath, recurse, pool);
	if(error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	//rebuild the property list
	error = Refresh();
	if(error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	svn_pool_destroy (pool);
	return S_OK;
}

STDMETHODIMP SVNProperties::Remove(ISVNProperty* prop, VARIANT_BOOL recurse)
{
	apr_pool_t *			pool;
	pool = svn_pool_create (NULL);

	const char*		pname_utf8;
	svn_error_t*	error = NULL;
	svn_string_t*	pval;

	BSTR			pname;
	BSTR			pv;
	prop->get_Name(&pname);
	prop->get_Value(&pv);
	CString propname = CString(pname);
	CString propvalue = CString(pv);
	::SysFreeString(pname);
	::SysFreeString(pv);

	pval = svn_string_create ((LPCTSTR)propvalue, pool);

	error = svn_utf_cstring_to_utf8 (&pname_utf8, (LPCTSTR)propname, NULL, pool);
	if(error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	error = svn_client_propset (pname_utf8, NULL, (LPCTSTR)filePath, recurse, pool);
	if(error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	//rebuild the property list
	error = Refresh();
	if(error != NULL)
	{
		svn_pool_destroy (pool);
		return Error(error->message,0,NULL,this->GetObjectCLSID(),E_FAIL);
	}

	svn_pool_destroy (pool);
	return S_OK;
}
