// SVNProperties.h : Declaration of the SVNProperties

#pragma once
#include "resource.h"       // main symbols

#include "apr_general.h"
#include "svn_pools.h"
#include "svn_client.h"
#include "svn_sorts.h"
#include "svn_path.h"
#include "svn_utf.h"

#include "SVNProperty.h"


// ISVNProperties
[
	object,
	uuid("DD79FA61-A08A-4958-915E-73E323E239A1"),
	dual,	helpstring("ISVNProperties Interface"),
	pointer_default(unique)
]
__interface ISVNProperties : IDispatch
{
	[id(1), helpstring("method Create")] HRESULT Create([in] BSTR path);
	[propget, id(2), helpstring("property Count")] HRESULT Count([out, retval] LONG* pVal);
	[id(3), helpstring("method Item")] HRESULT Item([in] LONG index, [out,retval] IDispatch** prop);
	[id(4), helpstring("method Add")] HRESULT Add([in] ISVNProperty* prop, [in] VARIANT_BOOL recurse);
	[id(5), helpstring("method Remove")] HRESULT Remove([in] ISVNProperty* prop, [in] VARIANT_BOOL recurse);
};




/**
 * \ingroup SVNCOM
 *
 * This class represents all the properties of a file or
 * directory. 
 *
 * \version 1.0
 * first version
 *
 * \date 09-27-2002
 *
 * \author kueng
 *
 * \par license
 * This code is absolutely free to use and modify. The code is provided "as is" with
 * no expressed or implied warranty. The author accepts no liability if it causes
 * any damage to your computer, causes your pet to fall ill, increases baldness
 * or makes your car start emitting strange noises when you start it up.
 * This code has no bugs, just undocumented features!
 * 
 * \todo 
 *
 * \bug 
 *
 */
[
	coclass,
	threading("apartment"),
	vi_progid("SVNCOM.SVNProperties"),
	progid("SVNCOM.SVNProperties.1"),
	version(1.0),
	uuid("19C8EADD-B52A-40C3-B6AA-9863941F52C6"),
	helpstring("SVNProperties Class")
]
class ATL_NO_VTABLE SVNProperties : 
	public ISVNProperties
{
public:
	SVNProperties()
	{
	}


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		pool = svn_pool_create (NULL);
		return S_OK;
	}
	
	void FinalRelease() 
	{
		svn_pool_destroy (pool);
	}

private:
	CString					filePath;
	apr_array_header_t*		props;
	int						propCount;
	int						currentProp;
	apr_pool_t*				pool;


	/**
	 * Builds the properties (again) and fills the apr_array_header_t structure.
	 * \return the svn error structure
	 */
	svn_error_t*			Refresh();

public:

	/**
	 * Creates a SVNProperties object. Use this if you didn't got the
	 * object from the SVN object with GetProperties().
	 * \param path the path to a file or directory to get the properties from.
	 */
	STDMETHOD(Create)(BSTR path);
	/**
	 * Get the number of properties found. Implemented as a COM-Property.
	 */
	STDMETHOD(get_Count)(LONG* pVal);
	/**
	 * Creates a SVNProperty object for the indexed property.
	 * \param index the property number to get
	 * \param prop the returned SVNProperty object
	 */
	STDMETHOD(Item)(LONG index, IDispatch** prop);
	/**
	 * Adds a new property to the file/directory. The file/directory is set
	 * with the Create() method or with the SVN.GetProperties() method.
	 * \param prop an SVNProperty object to add to the properties
	 * \param recurse TRUE if all files (if given a directory) should 
	 *						get the new property.
	 */
	STDMETHOD(Add)(ISVNProperty* prop, VARIANT_BOOL recurse);
	/**
	 * Removes a property from the file/directory. The file/directory is set
	 * with the Create() method or with the SVN.GetProperties() method.
	 * \param prop the property to delete
	 * \param recurse TRUE if all files (if given a directory) should
	 *						have the property removed.
	 */
	STDMETHOD(Remove)(ISVNProperty* prop, VARIANT_BOOL recurse);
};

