// SVNStatus.h : Declaration of the SVNStatus

#pragma once
#include "resource.h"       // main symbols

/**
 * SVNStatus enumerator.
 */
[export]
typedef enum 
{
    status_none = 1,    ///< does not exist
    status_unversioned, ///< is not a versioned thing in this wc 
    status_normal,      ///< exists, but uninteresting. 
    status_added,       ///< is scheduled for additon 
    status_absent,      ///< under v.c., but is missing 
    status_deleted,     ///< scheduled for deletion 
    status_replaced,    ///< was deleted and then re-added 
    status_modified,    ///< text or props have been modified 
    status_merged,      ///< local mods received repos mods 
    status_conflicted,  ///< local mods received conflicting repos mods 
    status_obstructed   ///< an unversioned resource is in the way of the versioned resource
} svn_statuses;

/**
 * SVN schedule enum.
 */
[export]
typedef enum 
{
  schedule_normal,       ///< Nothing special here 
  schedule_add,          ///< Slated for addition 
  schedule_delete,       ///< Slated for deletion 
  schedule_replace       ///< Slated for replacement (delete + add) 
} svn_schedule;


[
	object,
	uuid("C8BE8DCB-8D15-4979-A8EE-06A43B320ED2"),
	dual,	helpstring("ISVNStatus Interface"),
	pointer_default(unique)
]
__interface ISVNStatus : IDispatch
{
	[propget, id(1), helpstring("the name of the status entry")] HRESULT name([out, retval] BSTR* pVal);
	[propget, id(2), helpstring("base revision")] HRESULT revision([out, retval] LONG* pVal);
	[propget, id(3), helpstring("url in repository")] HRESULT url([out, retval] BSTR* pVal);
	[propget, id(4), helpstring("canonical repository url")] HRESULT repository([out, retval] BSTR* pVal);
	[propget, id(5), helpstring("scheduling (add, delete, replace ...)")] HRESULT schedule([out, retval] svn_schedule* pVal);
	[propget, id(6), helpstring("in a copied state")] HRESULT copied([out, retval] VARIANT_BOOL* pVal);
	[propget, id(7), helpstring("deleted, but parent rev lags behind")] HRESULT deleted([out, retval] VARIANT_BOOL* pVal);
	[propget, id(8), helpstring("copyfrom revision")] HRESULT copyfrom_rev([out, retval] LONG* pVal);
	[propget, id(9), helpstring("copyfrom location")] HRESULT copyfrom_url([out, retval] BSTR* pVal);
	[propget, id(10), helpstring("old version of conflicted file")] HRESULT conflict_old([out, retval] BSTR* pVal);
	[propget, id(11), helpstring("new version of conflicted file")] HRESULT conflict_new([out, retval] BSTR* pVal);
	[propget, id(12), helpstring("working version of conflicted file")] HRESULT conflict_wrk([out, retval] BSTR* pVal);
	[propget, id(13), helpstring("last up-to-date time for text contents")] HRESULT text_time([out, retval] DATE* pVal);
	[propget, id(14), helpstring("last up-to-date time for properties")] HRESULT prop_time([out, retval] DATE* pVal);
	[propget, id(15), helpstring("last revision this was changed")] HRESULT cmt_rev([out, retval] LONG* pVal);
	[propget, id(16), helpstring("last date this was changed")] HRESULT cmt_date([out, retval] DATE* pVal);
	[propget, id(17), helpstring("last commit author of this item")] HRESULT cmt_author([out, retval] BSTR* pVal);
	[propget, id(18), helpstring("status of the text entry")] HRESULT text_status([out, retval] svn_statuses* pVal);
	[propget, id(19), helpstring("status of the properties entry")] HRESULT prop_status([out, retval] svn_statuses* pVal);
	[propget, id(20), helpstring("status of the text entry in the repository")] HRESULT repos_text_status([out, retval] svn_statuses* pVal);
	[propget, id(21), helpstring("status of the properties entry in the repository")] HRESULT repos_prop_status([out, retval] svn_statuses* pVal);
	[propget, id(22), helpstring("item update was interrupted")] HRESULT locked([out, retval] VARIANT_BOOL* pVal);
};

/**
 * \ingroup SVNCOM
 *
 * Status object. A simple struct implemented as a COM object
 * with properties for simpler use.
 * All properties are read-only!
 *
 * \version 1.0
 * first version
 *
 * \date 09-27-2002
 *
 * \author kueng
 *
 * \par license
 * This code is absolutely free to use and modify. The code is provided "as is" with
 * no expressed or implied warranty. The author accepts no liability if it causes
 * any damage to your computer, causes your pet to fall ill, increases baldness
 * or makes your car start emitting strange noises when you start it up.
 * This code has no bugs, just undocumented features!
 * 
 * \todo 
 *
 * \bug 
 *
 */
[
	coclass,
	threading("apartment"),
	vi_progid("SVNCOM.SVNStatus"),
	progid("SVNCOM.SVNStatus.1"),
	version(1.0),
	uuid("1E71A73A-450F-473C-8565-806075A767F3"),
	helpstring("SVNStatus Class")
]
class ATL_NO_VTABLE SVNStatus : 
	public ISVNStatus
{
public:
	SVNStatus()
	{
	}


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
	}

public:
	svn_statuses	text_status;
	svn_statuses	prop_status;
	BOOL			locked;
	svn_statuses	repo_text_status;
	svn_statuses	repo_prop_status;

	CString			name;
	LONG			revision;
	CString			url;
	CString			repository;

	/* State information */
	svn_schedule	schedule;
	BOOL			copied;
	BOOL			deleted;
	CString			copyfrom_url;
	LONG			copyfrom_rev;

	CString			conflict_old;
	CString			conflict_new;
	CString			conflict_wrk;
	//const char *prejfile;          /* property reject file */

	/* Timestamps (0 means no information available) */
	DATE			text_time;
	DATE			prop_time;
	/* Checksum.  Optional; can be NULL for backwards compatibility. */
	//const char *checksum;          /* base64-encoded checksum for the
									//untranslated text base file. */

	/* "Entry props" */
	LONG			cmt_rev;
	DATE			cmt_date;
	CString			cmt_author;

	/// the name of the status entry
	STDMETHOD(get_name)(BSTR* pVal);
	/// base revision
	STDMETHOD(get_revision)(LONG* pVal);
	/// url in repository
	STDMETHOD(get_url)(BSTR* pVal);
	/// canonical repository url
	STDMETHOD(get_repository)(BSTR* pVal);
	/// scheduling (add, delete, replace,...) see svn_schedule enum.
	STDMETHOD(get_schedule)(svn_schedule* pVal);
	/// in a copied state (true) or not (false)
	STDMETHOD(get_copied)(VARIANT_BOOL* pVal);
	/// deleted, but parent rev lags behind
	STDMETHOD(get_deleted)(VARIANT_BOOL* pVal);
	/// copyfrom revision
	STDMETHOD(get_copyfrom_rev)(LONG* pVal);
	/// copyfrom location
	STDMETHOD(get_copyfrom_url)(BSTR* pVal);
	/// old version of conflicted file
	STDMETHOD(get_conflict_old)(BSTR* pVal);
	/// new version of conflicted file
	STDMETHOD(get_conflict_new)(BSTR* pVal);
	/// working version of conflicted file
	STDMETHOD(get_conflict_wrk)(BSTR* pVal);
	/// last up-to-date time for text contents
	STDMETHOD(get_text_time)(DATE* pVal);
	/// last up-to-date time for properties
	STDMETHOD(get_prop_time)(DATE* pVal);
	/// last revision this file/directory was changed
	STDMETHOD(get_cmt_rev)(LONG* pVal);
	/// last date this file/directory was changed
	STDMETHOD(get_cmt_date)(DATE* pVal);
	/// last commit author of this item
	STDMETHOD(get_cmt_author)(BSTR* pVal);
	/// status of the text entry
	STDMETHOD(get_text_status)(svn_statuses* pVal);
	/// status of the text entry in the repository
	STDMETHOD(get_prop_status)(svn_statuses* pVal);
	/// status of the properties entry
	STDMETHOD(get_repos_text_status)(svn_statuses* pVal);
	/// status of the properties entry in the repository
	STDMETHOD(get_repos_prop_status)(svn_statuses* pVal);
	/// item update was interrupted
	STDMETHOD(get_locked)(VARIANT_BOOL* pVal);
};

